import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { Store, select } from '@ngrx/store';

import { NGXLogger } from 'ngx-logger';
import { userNotification } from './actions';
import { map, tap, withLatestFrom } from 'rxjs/operators';

import { UserInfoUpdateType } from '@ucap-webmessenger/protocol-info';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import * as AuthenticationStore from '@app/store/account/authentication';

@Injectable()
export class Effects {
  userNotification$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(userNotification),
        withLatestFrom(
          this.store.pipe(
            select(
              (state: any) =>
                state.account.authentication.loginRes as LoginResponse
            )
          )
        ),
        // map(action => action.noti),
        tap(([action, loginRes]) => {
          if (action.noti.type === UserInfoUpdateType.Image) {
            const urlend = loginRes.userInfo.profileImageFile.indexOf(
              'ProfileImage'
            );
            let rootUrl;

            if (action.noti.info.indexOf('http') < 0) {
              rootUrl = loginRes.userInfo.profileImageFile.substring(
                0,
                urlend + 1
              );
            }
            const userInfo = {
              ...loginRes.userInfo,
              // profileImageFile: res.profileSubDir
              profileImageFile: action.noti.info
            };

            this.store.dispatch(
              AuthenticationStore.updateLoginRes({
                loginRes: {
                  ...loginRes,
                  userInfo
                }
              })
            );
          }
        })
      );
    },
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private logger: NGXLogger
  ) {}
}
